/*
 * Copyright (c) 2011 Stefan Bolte <portix@gmx.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <locale.h>
#include <termios.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

#ifndef NAME
#define NAME "xshowkey"
#endif
#ifndef VERSION
#define VERSION "0.2"
#endif


#define BUFFER_SIZE  512 

void 
help(int status) {
    printf("Usage: %s [-h | -v]\n", NAME);
    exit(status);
}
void 
version() {
    printf("%s %s\n", NAME, VERSION);
    exit(EXIT_SUCCESS);
}

int main(int argc, char **argv) {
    XEvent e;
    Display *dpy; 
    Window focus;
    int ret, flags, n;
    char buffer[BUFFER_SIZE];
    KeySym ks, last = XK_VoidSymbol;
    char *keysym;
    struct termios attr;

    setlocale(LC_ALL, "");

    if (argc > 1) 
    {
        if (argv[1][0] == '-') {
            switch (argv[1][1]) { 
                case 'h' : help(EXIT_SUCCESS); 
                case 'v' : version(); 
                default  : help(EXIT_FAILURE);
            }
        }
        else {
            help(EXIT_FAILURE);
        }
    }
    if ( ! (dpy = XOpenDisplay(NULL))) 
    {
        fprintf(stderr, "Cannot open display.\n");
        return EXIT_FAILURE;
    }
    XGetInputFocus(dpy, &focus, &ret);
    XSelectInput(dpy, focus, KeyPressMask);

    tcgetattr(0, &attr);
    flags = attr.c_lflag;
    attr.c_lflag &= ~(ECHO);
    tcsetattr(0, TCSANOW, &attr);

    printf("press any key ('Escape' twice exits)...\n\n");
    while (1) 
    {
        XNextEvent(dpy, &e);
        if (e.type == KeyPress) {
            n = XLookupString(&e.xkey, buffer, 256, &ks, NULL);
            buffer[n < 0 ? 0 : n > BUFFER_SIZE - 1 ? BUFFER_SIZE - 1 : n] = '\0';
            keysym = XKeysymToString(ks);

            if (last == XK_Escape && ks == XK_Escape) 
            {
                attr.c_lflag = flags;
                tcsetattr(0, TCSAFLUSH, &attr);
                XCloseDisplay(dpy);
                return EXIT_SUCCESS;
            }
            else 
            {
                buffer[n > 256 ? 256 : n < 0 ? 0 : n] = '\0';
                if (*buffer == '\r' || *buffer == '\t' || *buffer == '\n' || *buffer == '\f' || *buffer == 127) 
                    *buffer = '\0';
                printf("Symbol: %s \tkeycode: %d \tkeysym: %s\n", buffer, e.xkey.keycode, keysym);
            }
            last = ks;
        }
    }
    return EXIT_SUCCESS;
}
